!function($,D,window,document,undefined) {
  "use strict";

  Drupal.behaviors.dismissible_message_bar = {
    attach: function (context, settings) {

      /**
       * Retrieve DMB Notification Settings.
       *
       * @return {Array}
       *   Array of defined DMB Settings.
       */
      const getNotificationSettings = function () {
        if (typeof(settings.dmbNotifications) !== 'undefined' && settings.dmbNotifications) {
          return settings.dmbNotifications;
        }
        return null;
      };

      /**
       * Retrieve DMB Notification Entity data.
       *
       * @return {Array}
       *   Array of defined DMB Entities.
       */
      const getNotificationEntities = function () {
        if (typeof(settings.dmbNotificationEntities) !== 'undefined' && settings.dmbNotificationEntities) {
          return settings.dmbNotificationEntities;
        }
        return null;
      };

      /**
       * Close notification functionality
       * @param $notification
       *   Notification element to be closed.
       */
      const closeNotification = function ($notification) {
        let notification_id = $notification.attr('dmb-notification-id'),
          cookie_expiration = $notification.attr('dmb-cookie-expiration'),
          accepted_notifications = $.cookie('dismissible_message_bar');

        if (!accepted_notifications) {
          accepted_notifications = notification_id;
        } else {
          accepted_notifications = accepted_notifications.split('.');
          if (!accepted_notifications.includes(notification_id)) {
            accepted_notifications.push(notification_id);
          }
          accepted_notifications = accepted_notifications.join('.');
        }

        // Set the notification cookie with updated values
        const setAcceptedCookie = function () {
          $.cookie('dismissible_message_bar', accepted_notifications, {
            expires: Number(cookie_expiration),
            path: '/'
          });
        };

        // Only set cookie if cookie-off class doesn't exist.
        if(!$('.cookie-off').length) {
          setAcceptedCookie();
        }

        $notification.slideUp();
        return false;
      }

      /**
       * Inject Notifications into their respective wrappers.
       */
      const injectNotifications = function (notificationSettings, $notificationWrapper, requestData) {
        // Inject notifications.
        if (notificationSettings !== null && $notificationWrapper !== null) {

          let data = getNotificationEntities();
          let content = '';
          for (let key in data) {
            // Skip loop if the property is from prototype.
            if (!data.hasOwnProperty(key)) {
              continue;
            }

            if ($notificationWrapper.length > 0) {
              if (data[key].content !== null) {
                let showNotification = (data[key].sitewide === 1 || data[key].sitewide === "1");

                // Check content types.
                if (data[key].content_types !== null && data[key].content_types.length !== 0 && notificationSettings.contentType !== '') {
                  if (data[key].content_types.includes(notificationSettings.contentType)) {
                    showNotification = true;
                  }
                }

                // Check path limitations.
                if (data[key].sitewide !== null && (data[key].sitewide === 0 || data[key].sitewide === "0")) {
                  if (data[key].path_limit !== null && window.location.pathname !== '') {
                    for (let paths in data[key].path_limit) {
                      if (!data[key].path_limit.hasOwnProperty(paths)) {
                        continue;
                      }

                      let search_path = data[key].path_limit[paths];
                      let is_not = false
                      if (search_path.indexOf("!") === 0) {
                        // This is a NOT path.
                        is_not = true;
                        search_path = search_path.slice(1);
                      }

                      if (search_path === window.location.pathname) {
                        showNotification = !is_not;
                        break;
                      }
                      else {
                        if (search_path.indexOf("*") !== -1) {
                          // This is a wildcard search. Replace the * with a .*
                          search_path.replace(/\*/g, ".*");
                          if (window.location.pathname.search(search_path) !== -1) {
                            showNotification = true;
                            break;
                          }
                        }
                      }
                    }
                  }
                }

                // Check date limitations
                if (showNotification) {
                  let nowDate = new Date();
                  if (data[key].start_time !== null && data[key].start_time !== "") {
                    let startDate = new Date(data[key].start_time);
                    if (startDate instanceof Date && !isNaN(startDate)) {
                      if (startDate.valueOf() > nowDate.valueOf()) {
                        // This notification should not appear yet.
                        showNotification = false
                      }
                    }
                  }
                  if (showNotification && data[key].end_time !== null && data[key].end_time !== "") {
                    let endDate = new Date(data[key].end_time);
                    if (endDate instanceof Date && !isNaN(endDate)) {
                      if (nowDate.valueOf() > endDate.valueOf()) {
                        // This notification has passed.
                        showNotification = false;
                      }
                    }
                  }
                }

                if (showNotification) {
                  content += data[key].content
                }
              }
            }
          }

          if (content !== '') {
            $notificationWrapper.once('inject-dmb-notifications').append(content);
            $notificationWrapper.once('inject-dmb-notifications-trigger').trigger('injectDMBNotifications');
          }

        }

      };

      // Initialize notifications.
      let settingsObject = getNotificationSettings();
      let notificationSettingsArray = Object.values(settingsObject);
      for (let i in notificationSettingsArray) {
        let notificationSettings = notificationSettingsArray[i];
        let $notificationWrapper = null,
          $overlayWrapper = null,
          requestData = {
            contentType: '',
            contentPath: '',
            notificationType: '',
          };

        if (typeof(notificationSettings.notificationWrapper) !== 'undefined' && notificationSettings.notificationWrapper) {
          $notificationWrapper = $(notificationSettings.notificationWrapper);
        }

        if (typeof(notificationSettings.overlayWrapper) !== 'undefined' && notificationSettings.overlayWrapper) {
          $overlayWrapper = $(notificationSettings.overlayWrapper);
        }

        // These parameters have to be saved and passed into the handler.
        if (typeof(notificationSettings.contentType) !== 'undefined' && notificationSettings.contentType) {
          requestData.contentType = notificationSettings.contentType;
        }

        if (typeof(notificationSettings.contentPath) !== 'undefined' && notificationSettings.contentPath) {
          requestData.contentPath = notificationSettings.contentPath;
        }

        if (typeof(notificationSettings.notificationType) !== 'undefined' && notificationSettings.notificationType) {
          requestData.notificationType = notificationSettings.notificationType;
        }

        injectNotifications(notificationSettings, $notificationWrapper, requestData);

        if ($notificationWrapper !== null) {

          $notificationWrapper.on('injectDMBNotifications', function () {
            let $autoDismiss = $notificationWrapper.find('[dmb-auto-dismiss="1"]');
            $autoDismiss.each(function () {
              let time = $(this).attr('dmb-dismiss-time') * 1000;
              let $notification = $(this);
              setTimeout(function () {
                closeNotification($notification);
              }, time);
            });

            $('.dmb-notification .close-dmb-notification').once().on('click touchstart', function (e) {
              e.preventDefault();
              closeNotification($(this).parent());
            });
          });
        }

      }

    }
  };

}(jQuery,Drupal,window,document,'');
